==> boostcrm-widget-1.0.2.tgz: package/es/connections/websocket.js <==
import _regeneratorRuntime from 'babel-runtime/regenerator';
import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _extends from 'babel-runtime/helpers/extends';
import _debounce from 'lodash/debounce';
import _get from 'lodash/get';
import * as io from 'socket.io-client';

import urljoin from 'url-join';
import { getLSVariable, setLSVariable } from '../commons/utils/local-storage';
import { LOCAL_STORAGE_VARIABLES, getLocalStorageVariables } from '../commons/constants/local-storage';
import { getTenantInfo } from '../commons/tenant-info';
import getClientInfo from '../commons/utils/get-client-info';
import { GROUP, TYPE } from '../commons/constants/websocket';
var queuedMessages = [];

var wsClient = null;
var getWSHost = function getWSHost(apiHost) {
  if (apiHost.indexOf('localhost') >= 0 || apiHost.indexOf('127.0.0.1') >= 0) return apiHost;
  return apiHost.replace('https://', 'https://ws.').replace('http://', 'http://ws.');
};

var sendOrQueueMessage = function sendOrQueueMessage(wsMessageType, payload) {
  // console.log(`wsMessageType ${wsMessageType}:`, JSON.stringify(payload));
  var tenantInfo = getTenantInfo();
  // console.log("tenantInfo:", JSON.stringify(tenantInfo));
  if (wsClient && wsClient.socket && wsClient.socket.connected && tenantInfo.conversationId) {
    // Send all messages in queue first if available
    // console.log("Send all messages in queue");
    while (queuedMessages.length > 0) {
      console.log('send message from queue');
      var queuedMessage = queuedMessages.shift();
      queuedMessage && wsClient.emitMessage(queuedMessage.wsMessageType, _extends({}, queuedMessage.payload, {
        tenantInfo: tenantInfo
      }));
    }

    wsClient.emitMessage(wsMessageType, _extends({}, payload, {
      tenantInfo: tenantInfo
    }));
  } else {
    console.log('enqueue the message: ', { wsMessageType: wsMessageType, payload: payload });
    // Add to queue
    queuedMessages.push({ wsMessageType: wsMessageType, payload: payload });
    // Enable retry
    retrySendQueuedMessages();
  }
};

var retrySendQueuedMessages = function retrySendQueuedMessages(channelId) {
  console.log('retrySendQueuedMessages');
  var interval = setInterval(function () {
    var tenantInfo = getTenantInfo();
    if (wsClient && wsClient.connected && tenantInfo.conversationId) {
      console.log('send messsages from queues: ', queuedMessages.length);
      while (queuedMessages.length > 0) {
        var queuedMessage = queuedMessages.shift();
        queuedMessage && wsClient.emitMessage(queuedMessage.wsMessageType, _extends({}, queuedMessage.payload, {
          tenantInfo: tenantInfo
        }));
      }
      clearInterval(interval);
    }
  }, 1000);
};

var debounceReconnect = _debounce(function (ws, onConnected) {
  ws.connected && onConnected && onConnected();
}, 3000);

var WebSocket = function () {
  function WebSocket(host, token, isFromSimulator, channelKey) {
    var _this = this;

    _classCallCheck(this, WebSocket);

    this.onMessageHandler = null;
    this.onInitWSSuccess = null;
    this.onStartConversationSuccess = null;
    this.isConnected = false; // Important! Don't depend on `wsClient.socket.connected`. This field could be true before the event onConnect
    this.isReconnecting = false;
    var WS_HOST = urljoin(getWSHost(host), '/chatbox');
    this.socket = io(WS_HOST, {
      transports: ['websocket', 'polling'],
      query: {
        token: '' + token
      },
      forceNew: true,
      reconnection: true,
      reconnectionDelay: 2000,
      reconnectionDelayMax: 5000,
      reconnectionAttempts: 10,
      timeout: 30000
    });

    this.socket.on('connect', _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
      var channelId, tenantId, conversationId;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              console.log('on connect');
              _this.connected = true;
              // Check if it is a old conversation
              channelId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CHANNEL_ID, channelKey));
              tenantId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.TENANT_ID, channelKey));
              conversationId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey));

              if (!(isFromSimulator || !conversationId || conversationId === 'undefined')) {
                _context.next = 18;
                break;
              }

              _context.t0 = _this.socket;
              _context.t1 = channelId;
              _context.t2 = tenantId;
              _context.next = 11;
              return getClientInfo();

            case 11:
              _context.t3 = _context.sent;
              _context.t4 = isFromSimulator;
              _context.t5 = {
                channelId: _context.t1,
                tenantId: _context.t2,
                visitor: _context.t3,
                isFromSimulator: _context.t4
              };

              _context.t6 = function (response) {
                conversationId = _get(response, 'id');
                setLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey), conversationId);

                _this.onStartConversationSuccess && _this.onStartConversationSuccess(conversationId);
              };

              _context.t0.emit.call(_context.t0, 'new-conversation', _context.t5, _context.t6);

              _context.next = 28;
              break;

            case 18:
              _context.t7 = _this.socket;
              _context.t8 = channelId;
              _context.t9 = tenantId;
              _context.t10 = conversationId;
              _context.next = 24;
              return getClientInfo();

            case 24:
              _context.t11 = _context.sent;
              _context.t12 = {
                channelId: _context.t8,
                tenantId: _context.t9,
                conversationId: _context.t10,
                visitor: _context.t11
              };

              _context.t13 = function (response) {
                // console.log("response: ", response);
                conversationId = _get(response, 'id');
                if (conversationId) {
                  setLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey), conversationId);
                  _this.onStartConversationSuccess && _this.onStartConversationSuccess(conversationId);
                }
              };

              _context.t7.emit.call(_context.t7, 'old-conversation', _context.t12, _context.t13);

            case 28:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    })));

    this.socket.on('error', function (err) {
      console.log('error on socket:', err);
    });

    this.socket.on('disconnect', function (reason) {
      _this.connected = false;
      console.log('WS:socket disconnect:', reason);
      if (reason === 'io server disconnect') {
        _this.socket.connect();
        return;
      }
    });

    this.socket.on('message', function (data) {
      _this.onMessageHandler && _this.onMessageHandler(data);
    });

    this.socket.on('ping', function (data) {
      if (data === 'ping') {
        _this.emitMessage('message', 'pong');
      }
    });

    this.socket.on('reconnecting', function (attemptNumber) {
      console.log('WS:on reconnecting, total attemptNumber: ', attemptNumber);
      _this.onReconnecting && _this.onReconnecting();
    });

    this.socket.on('reconnect_error', function (error) {
      console.log('WS:on reconnect_error: ');
    });

    this.socket.on('reconnect_failed', function () {
      _this.connected = false;
      console.log('WS:on reconnect_failed: ');
      _this.onConnectFailed && _this.onConnectFailed();
    });

    this.socket.on('connect_error', function (error) {
      _this.connected = false;
      console.log('WS:on connect_error: ');
    });

    this.socket.on('connect_timeout', function (timeout) {
      console.log('WS:on connect_timeout: ', timeout);
      _this.onConnectFailed && _this.onConnectFailed();
    });

    this.socket.on('reconnect', function (attemptNumber) {
      _this.connected = true;
      console.log('WS:on reconnect after attemptNumber: ', attemptNumber);
      debounceReconnect(_this.socket, _this.onConnected);
    });
  }

  WebSocket.prototype.emitMessage = function emitMessage(type, data) {
    if (!this.connected) {
      console.log('emit message when no connection: ', data);
      // Debug root cause show the connection interrupted
      // this.onConnectFailed && this.onConnectFailed();
      return;
    }
    this.socket.emit(type, data);
  };

  WebSocket.prototype.onReceiveMessage = function onReceiveMessage(handler) {
    // console.log("set onReceiveMessageHandler");
    this.onMessageHandler = handler;
  };

  WebSocket.prototype.onStartConversation = function onStartConversation(handler) {
    this.onStartConversationHandler = handler;
  };

  WebSocket.prototype.onResumeConversation = function onResumeConversation(handler) {
    this.onResumeConversationHandler = handler;
  };

  WebSocket.prototype.closeConnection = function closeConnection() {
    if (this.socket && this.socket.close) {
      this.socket.close();
    }
  };

  WebSocket.prototype.disconnect = function disconnect() {
    this.socket.disconnect();
    this.onConnectFailed && this.onConnectFailed();
  };

  WebSocket.prototype.connect = function connect() {
    this.socket.connect();
    this.onConnected && this.onConnected();
  };

  WebSocket.prototype.isWSConnected = function isWSConnected() {
    return this.socket.connected;
  };

  return WebSocket;
}();

export function initWSClient(host, token, channelId, isFromSimulator, onInitWSSuccess, onStartConversationSuccess, onConnectFailed, onReconnecting, onConnected) {
  // console.log("initWSClient");
  if (wsClient) return wsClient;
  wsClient = new WebSocket(host, token, isFromSimulator, channelId);
  wsClient.onStartConversationSuccess = onStartConversationSuccess;
  wsClient.onInitWSSuccess = onInitWSSuccess;
  wsClient.onConnectFailed = onConnectFailed;
  wsClient.onReconnecting = onReconnecting;
  wsClient.onConnected = onConnected;
  return wsClient;
}

export function closeWSClient() {
  if (wsClient) {
    wsClient.closeConnection();
    wsClient = null;
  }
}

export function disconnectWSClient() {
  if (!wsClient) return null;
  wsClient.disconnect();
}

export function reconnectWSClient() {
  if (!wsClient) return null;
  !wsClient.isWSConnected() && wsClient.connect();
}

export function sendMessage(message) {
  sendOrQueueMessage('message', message);
}

export function sendWebsiteEvent(eventType, payload) {
  sendOrQueueMessage('website-event', {
    event: { type: eventType, payload: payload }
  });
}

export function triggerStartingPoint(id) {
  sendOrQueueMessage('message', {
    group: GROUP.MESSAGE,
    type: TYPE.MESSAGE.TRIGGER_STARTING_POINT,
    payload: {
      id: id
    }
  });
}

export function setMessageHanlder(handler) {
  if (!wsClient) {
    // console.log("Could not set message handler");
    return;
  }
  wsClient.onReceiveMessage(handler);
}

export function setStartConversationHandler(handler) {
  if (!wsClient) return;
  wsClient.onStartConversationHandler(handler);
}

export function setResumeConversationHandler(handler) {
  if (!wsClient) return;
  wsClient.onResumeConversationHandler(handler);
}

==> boostcrm-widget-1.0.8.tgz: package/es/connections/websocket.js <==
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _extends from "@babel/runtime/helpers/extends";
import _debounce from "lodash/debounce";
import _get from "lodash/get";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import * as io from 'socket.io-client';
import urljoin from 'url-join';
import { getLSVariable, setLSVariable } from '../commons/utils/local-storage';
import { LOCAL_STORAGE_VARIABLES, getLocalStorageVariables } from '../commons/constants/local-storage';
import { getTenantInfo } from '../commons/tenant-info';
import getClientInfo from '../commons/utils/get-client-info';
import { GROUP, TYPE } from '../commons/constants/websocket';
var queuedMessages = [];
var wsClient = null;

var getWSHost = function getWSHost(apiHost) {
  if (apiHost.indexOf('localhost') >= 0 || apiHost.indexOf('127.0.0.1') >= 0) return apiHost;
  return apiHost.replace('https://', 'https://ws.').replace('http://', 'http://ws.');
};

var sendOrQueueMessage = function sendOrQueueMessage(wsMessageType, payload) {
  // console.log(`wsMessageType ${wsMessageType}:`, JSON.stringify(payload));
  var tenantInfo = getTenantInfo(); // console.log("tenantInfo:", JSON.stringify(tenantInfo));

  if (wsClient && wsClient.socket && wsClient.socket.connected && tenantInfo.conversationId) {
    // Send all messages in queue first if available
    // console.log("Send all messages in queue");
    while (queuedMessages.length > 0) {
      console.log('send message from queue');
      var queuedMessage = queuedMessages.shift();
      queuedMessage && wsClient.emitMessage(queuedMessage.wsMessageType, _extends({}, queuedMessage.payload, {
        tenantInfo: tenantInfo
      }));
    }

    wsClient.emitMessage(wsMessageType, _extends({}, payload, {
      tenantInfo: tenantInfo
    }));
  } else {
    console.log('enqueue the message: ', {
      wsMessageType: wsMessageType,
      payload: payload
    }); // Add to queue

    queuedMessages.push({
      wsMessageType: wsMessageType,
      payload: payload
    }); // Enable retry

    retrySendQueuedMessages();
  }
};

var retrySendQueuedMessages = function retrySendQueuedMessages(channelId) {
  console.log('retrySendQueuedMessages');
  var interval = setInterval(function () {
    var tenantInfo = getTenantInfo();

    if (wsClient && wsClient.connected && tenantInfo.conversationId) {
      console.log('send messsages from queues: ', queuedMessages.length);

      while (queuedMessages.length > 0) {
        var queuedMessage = queuedMessages.shift();
        queuedMessage && wsClient.emitMessage(queuedMessage.wsMessageType, _extends({}, queuedMessage.payload, {
          tenantInfo: tenantInfo
        }));
      }

      clearInterval(interval);
    }
  }, 1000);
};

var debounceReconnect = _debounce(function (ws, onConnected) {
  ws.connected && onConnected && onConnected();
}, 3000);

var WebSocket = /*#__PURE__*/function () {
  function WebSocket(host, token, isFromSimulator, channelKey) {
    var _this = this;

    this.onMessageHandler = null;
    this.onInitWSSuccess = null;
    this.onStartConversationSuccess = null;
    this.isConnected = false; // Important! Don't depend on `wsClient.socket.connected`. This field could be true before the event onConnect

    this.isReconnecting = false;
    var WS_HOST = urljoin(getWSHost(host), '/chatbox');
    this.socket = io(WS_HOST, {
      transports: ['websocket', 'polling'],
      query: {
        token: "" + token
      },
      forceNew: true,
      reconnection: true,
      reconnectionDelay: 2000,
      reconnectionDelayMax: 5000,
      reconnectionAttempts: 10,
      timeout: 30000
    });
    this.socket.on('connect', /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
      var channelId, tenantId, conversationId;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              console.log('on connect');
              _this.connected = true; // Check if it is a old conversation

              channelId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CHANNEL_ID, channelKey));
              tenantId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.TENANT_ID, channelKey));
              conversationId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey));

              if (!(isFromSimulator || !conversationId || conversationId === 'undefined')) {
                _context.next = 18;
                break;
              }

              _context.t0 = _this.socket;
              _context.t1 = channelId;
              _context.t2 = tenantId;
              _context.next = 11;
              return getClientInfo();

            case 11:
              _context.t3 = _context.sent;
              _context.t4 = isFromSimulator;
              _context.t5 = {
                channelId: _context.t1,
                tenantId: _context.t2,
                visitor: _context.t3,
                isFromSimulator: _context.t4
              };

              _context.t6 = function (response) {
                conversationId = _get(response, 'id');
                setLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey), conversationId);
                _this.onStartConversationSuccess && _this.onStartConversationSuccess(conversationId);
              };

              _context.t0.emit.call(_context.t0, 'new-conversation', _context.t5, _context.t6);

              _context.next = 28;
              break;

            case 18:
              _context.t7 = _this.socket;
              _context.t8 = channelId;
              _context.t9 = tenantId;
              _context.t10 = conversationId;
              _context.next = 24;
              return getClientInfo();

            case 24:
              _context.t11 = _context.sent;
              _context.t12 = {
                channelId: _context.t8,
                tenantId: _context.t9,
                conversationId: _context.t10,
                visitor: _context.t11
              };

              _context.t13 = function (response) {
                // console.log("response: ", response);
                conversationId = _get(response, 'id');

                if (conversationId) {
                  setLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey), conversationId);
                  _this.onStartConversationSuccess && _this.onStartConversationSuccess(conversationId);
                }
              };

              _context.t7.emit.call(_context.t7, 'old-conversation', _context.t12, _context.t13);

            case 28:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    })));
    this.socket.on('error', function (err) {
      console.log('error on socket:', err);
    });
    this.socket.on('disconnect', function (reason) {
      _this.connected = false;
      console.log('WS:socket disconnect:', reason);

      if (reason === 'io server disconnect') {
        _this.socket.connect();

        return;
      }
    });
    this.socket.on('message', function (data) {
      _this.onMessageHandler && _this.onMessageHandler(data);
    });
    this.socket.on('ping', function (data) {
      if (data === 'ping') {
        _this.emitMessage('message', 'pong');
      }
    });
    this.socket.on('reconnecting', function (attemptNumber) {
      console.log('WS:on reconnecting, total attemptNumber: ', attemptNumber);
      _this.onReconnecting && _this.onReconnecting();
    });
    this.socket.on('reconnect_error', function (error) {
      console.log('WS:on reconnect_error: ');
    });
    this.socket.on('reconnect_failed', function () {
      _this.connected = false;
      console.log('WS:on reconnect_failed: ');
      _this.onConnectFailed && _this.onConnectFailed();
    });
    this.socket.on('connect_error', function (error) {
      _this.connected = false;
      console.log('WS:on connect_error: ');
    });
    this.socket.on('connect_timeout', function (timeout) {
      console.log('WS:on connect_timeout: ', timeout);
      _this.onConnectFailed && _this.onConnectFailed();
    });
    this.socket.on('reconnect', function (attemptNumber) {
      _this.connected = true;
      console.log('WS:on reconnect after attemptNumber: ', attemptNumber);
      debounceReconnect(_this.socket, _this.onConnected);
    });
  }

  var _proto = WebSocket.prototype;

  _proto.emitMessage = function emitMessage(type, data) {
    if (!this.connected) {
      console.log('emit message when no connection: ', data); // Debug root cause show the connection interrupted
      // this.onConnectFailed && this.onConnectFailed();

      return;
    }

    this.socket.emit(type, data);
  };

  _proto.onReceiveMessage = function onReceiveMessage(handler) {
    // console.log("set onReceiveMessageHandler");
    this.onMessageHandler = handler;
  };

  _proto.onStartConversation = function onStartConversation(handler) {
    this.onStartConversationHandler = handler;
  };

  _proto.onResumeConversation = function onResumeConversation(handler) {
    this.onResumeConversationHandler = handler;
  };

  _proto.closeConnection = function closeConnection() {
    if (this.socket && this.socket.close) {
      this.socket.close();
    }
  };

  _proto.disconnect = function disconnect() {
    this.socket.disconnect();
    this.onConnectFailed && this.onConnectFailed();
  };

  _proto.connect = function connect() {
    this.socket.connect();
    this.onConnected && this.onConnected();
  };

  _proto.isWSConnected = function isWSConnected() {
    return this.socket.connected;
  };

  return WebSocket;
}();

export function initWSClient(host, token, channelId, isFromSimulator, onInitWSSuccess, onStartConversationSuccess, onConnectFailed, onReconnecting, onConnected) {
  // console.log("initWSClient");
  if (wsClient) return wsClient;
  wsClient = new WebSocket(host, token, isFromSimulator, channelId);
  wsClient.onStartConversationSuccess = onStartConversationSuccess;
  wsClient.onInitWSSuccess = onInitWSSuccess;
  wsClient.onConnectFailed = onConnectFailed;
  wsClient.onReconnecting = onReconnecting;
  wsClient.onConnected = onConnected;
  return wsClient;
}
export function closeWSClient() {
  if (wsClient) {
    wsClient.closeConnection();
    wsClient = null;
  }
}
export function disconnectWSClient() {
  if (!wsClient) return null;
  wsClient.disconnect();
}
export function reconnectWSClient() {
  if (!wsClient) return null;
  !wsClient.isWSConnected() && wsClient.connect();
}
export function sendMessage(message) {
  sendOrQueueMessage('message', message);
}
export function sendWebsiteEvent(eventType, payload) {
  sendOrQueueMessage('website-event', {
    event: {
      type: eventType,
      payload: payload
    }
  });
}
export function triggerStartingPoint(id) {
  sendOrQueueMessage('message', {
    group: GROUP.MESSAGE,
    type: TYPE.MESSAGE.TRIGGER_STARTING_POINT,
    payload: {
      id: id
    }
  });
}
export function setMessageHanlder(handler) {
  if (!wsClient) {
    // console.log("Could not set message handler");
    return;
  }

  wsClient.onReceiveMessage(handler);
}
export function setStartConversationHandler(handler) {
  if (!wsClient) return;
  wsClient.onStartConversationHandler(handler);
}
export function setResumeConversationHandler(handler) {
  if (!wsClient) return;
  wsClient.onResumeConversationHandler(handler);
}
